@@ -1,3 +1,20 @@
+'''
+mzML is a standard rich XML-format for raw mass spectrometry data storage.
+Please refer to `psidev.info <http://www.psidev.info/index.php?q=node/257>`_
+for the detailed specification of the format and structure of mzML files.
+
+In addition to mzML, there is a wrapping format called ``indexedmzML``
+which adds an extra layer to the XML document, including pre-computed byte offsets
+for each ``<spectrum>`` and ``<chromatogram>`` element.
+
+To write ``mzML`` without an index use :class:`PlainMzMLWriter`, and for ``indexedmzML``
+use :class:`IndexedMzMLWriter`. Because so many tools rely on the index, :class:`IndexedMzMLWriter`
+is exported under the alias `MzMLWriter`. The interface for these two classes are the same,
+with :class:`IndexedMzMLWriter` having slightly more complex behavior on writing and when finishing
+the document, though you are able to alter the indexing behavior via :attr:`IndexedMzMLWriter.index_builder`
+or through inheritance.
+'''
+
 import numbers
 import warnings
 
@@ -74,7 +91,7 @@ class DocumentSection(ComponentDispatcher, XMLWriterMixin):
         self.toplevel.__enter__()
         return self
 
-    def end(self, exc_type, exc_value, traceback):
+    def end(self, exc_type=None, exc_value=None, traceback=None):
         self.toplevel.__exit__(exc_type, exc_value, traceback)
         self.writer.flush()
 
@@ -157,7 +174,7 @@ class IndexedmzMLSection(DocumentSection):
         self.inner = element(self.writer, MzML(**self.section_args))
         self.inner.__enter__()
 
-    def end(self, exc_type, exc_value, traceback):
+    def end(self, exc_type=None, exc_value=None, traceback=None):
         self.inner.__exit__(exc_type, exc_value, traceback)
         self.writer.flush()
         self.write_index()
@@ -176,7 +193,7 @@ class PlainMzMLWriter(ComponentDispatcher, XMLDocumentWriter):
     that they have access to a universal identity map for each element in the document,
     that map is centralized in this class.
 
-    MzMLWriter inherits from :class:`.ComponentDispatcher`, giving it a :attr:`context`
+    MzMLWriter inherits from :class:`~psims.mzml.components.ComponentDispatcher`, giving it a :attr:`context`
     attribute and access to all `Component` objects pre-bound to that context with attribute-access
     notation.
 
@@ -266,7 +283,7 @@ class PlainMzMLWriter(ComponentDispatcher, XMLDocumentWriter):
             A list or other iterable of :class:`str`, :class:`dict`, or \*Param-types which will
             be placed in the ``<fileContent>`` element.
         source_files : list
-            A list or other iterable of dict or :class:`~.SourceFile`-like objects
+            A list or other iterable of dict or :class:`~psims.mzml.components.SourceFile`-like objects
             to be placed in the ``<sourceFileList>`` element
         """
         self.state_machine.transition("file_description")
@@ -439,6 +456,9 @@ class PlainMzMLWriter(ComponentDispatcher, XMLDocumentWriter):
                  instrument_configuration_id=None, intensity_unit=DEFAULT_INTENSITY_UNIT):
         '''Create a new :class:`~.Spectrum` instance to be written.
 
+        This method does not immediately write and close the spectrum element, leaving it
+        open for modification and embedding.
+
         Parameters
         ----------
         mz_array: :class:`np.ndarray` of floats
@@ -468,8 +488,11 @@ class PlainMzMLWriter(ComponentDispatcher, XMLDocumentWriter):
             The compression type name to use. Defaults to `COMPRESSION_ZLIB`.
         encoding: dict, optional
             A mapping from array name to NumPy data types.
-        other_arrays: dict, optional
-            A mapping of array names to additional data arrays
+        other_arrays: list, optional
+            An iterable of array names to additional data arrays. Array names may either be
+            strings, :class:`Mapping` objects that define :class:`~.CVParam` or :class:`~.UserParam`,
+            or such paramter objects themselves. Use the latter two methods when defining arrays with
+            units.
         scan_params: list, optional
             A list of cvParams for the `scan` of this `spectrum`
         scan_window_list: list, optional
@@ -481,10 +504,16 @@ class PlainMzMLWriter(ComponentDispatcher, XMLDocumentWriter):
         Returns
         -------
         :class:`~.Spectrum`
+
+        See Also
+        --------
+        :meth:`write_spectrum`
+        :meth:`chromatogram`
+        :meth:`write_chromatogram`
         '''
         self.state_machine.expects_state("spectrum_list")
         if encoding is None:
-            encoding = {MZ_ARRAY: np.float64}
+            encoding = {MZ_ARRAY: np.float64, CHARGE_ARRAY: np.int32}
         if params is None:
             params = []
         else:
@@ -551,8 +580,12 @@ class PlainMzMLWriter(ComponentDispatcher, XMLDocumentWriter):
         for array_type, array in other_arrays:
             if array_type is None:
                 raise ValueError("array type can't be None")
+            if isinstance(array_type, Mapping):
+                array_name = array_type['name']
+            else:
+                array_name = array_type
             array_tag = self._prepare_array(
-                array, encoding=encoding[array_type], compression=compression, array_type=array_type,
+                array, encoding=encoding[array_name], compression=compression, array_type=array_type,
                 default_array_length=default_array_length, scope='spectrum')
             array_list.append(array_tag)
         array_list_tag = self.BinaryDataArrayList(array_list)
@@ -592,6 +625,8 @@ class PlainMzMLWriter(ComponentDispatcher, XMLDocumentWriter):
                        instrument_configuration_id=None, intensity_unit=DEFAULT_INTENSITY_UNIT):
         '''Write a :class:`~.Spectrum` with the provided data.
 
+        To create a spectrum element but not immediately close it off, see the :meth:`spectrum` method.
+
         Parameters
         ----------
         mz_array: :class:`np.ndarray` of floats
@@ -621,8 +656,11 @@ class PlainMzMLWriter(ComponentDispatcher, XMLDocumentWriter):
             The compression type name to use. Defaults to `COMPRESSION_ZLIB`.
         encoding: dict, optional
             A mapping from array name to NumPy data types.
-        other_arrays: dict, optional
-            A mapping of array names to additional data arrays
+        other_arrays: list, optional
+            An iterable of array names to additional data arrays. Array names may either be
+            strings, :class:`Mapping` objects that define :class:`~.CVParam` or :class:`~.UserParam`,
+            or such paramter objects themselves. Use the latter two methods when defining arrays with
+            units.
         scan_params: list, optional
             A list of cvParams for the `scan` of this `spectrum`
         scan_window_list: list, optional
@@ -687,8 +725,12 @@ class PlainMzMLWriter(ComponentDispatcher, XMLDocumentWriter):
             array_list.append(intensity_array_tag)
 
         for array_type, array in other_arrays:
+            if isinstance(array_type, Mapping):
+                array_name = array_type['name']
+            else:
+                array_name = array_type
             array_tag = self._prepare_array(
-                array, encoding=encoding[array_type], compression=compression, array_type=array_type,
+                array, encoding=encoding[array_name], compression=compression, array_type=array_type,
                 default_array_length=default_array_length, scope='chromatogram')
             array_list.append(array_tag)
         params.append(chromatogram_type)
@@ -750,10 +792,10 @@ class PlainMzMLWriter(ComponentDispatcher, XMLDocumentWriter):
         if isinstance(precursors, self.PrecursorList.type):
             return precursors
         elif isinstance(precursors, (dict)):
-            precursors = self.PrecursorList([self._prepare_precursor_information(
+            precursors = self.PrecursorList([self.prepare_precursor_information(
                 intensity_unit=intensity_unit, **precursors)])
         elif isinstance(precursors, PrecursorBuilder):
-            precursors = self.PrecursorList([self._prepare_precursor_information(
+            precursors = self.PrecursorList([self.prepare_precursor_information(
                 precursors,
                 intensity_unit=intensity_unit)])
         else:
@@ -763,7 +805,7 @@ class PlainMzMLWriter(ComponentDispatcher, XMLDocumentWriter):
                     packaged.append(p)
                 elif isinstance(p, dict):
                     packaged.append(
-                        self._prepare_precursor_information(
+                        self.prepare_precursor_information(
                             intensity_unit=intensity_unit, **p))
                 elif isinstance(p, PrecursorBuilder):
                     packaged.append(
@@ -772,10 +814,10 @@ class PlainMzMLWriter(ComponentDispatcher, XMLDocumentWriter):
             precursors = self.PrecursorList(packaged)
         return precursors
 
-    def _prepare_precursor_information(self, mz=None, intensity=None, charge=None, spectrum_reference=None, activation=None,
+    def prepare_precursor_information(self, mz=None, intensity=None, charge=None, spectrum_reference=None, activation=None,
                                        isolation_window_args=None, params=None,
                                        intensity_unit=DEFAULT_INTENSITY_UNIT, scan_id=None, external_spectrum_id=None,
-                                       source_file_reference=None):
+                                       source_file_reference=None, **kwargs):
         '''Prepare a :class:`Precursor` element from disparate data structures.
 
         Parameters
@@ -788,14 +830,14 @@ class PlainMzMLWriter(ComponentDispatcher, XMLDocumentWriter):
             The charge state of the first seelcted ion
         spectrum_reference: str, optional
             The `id` of the prescursor `<spectrum>` for this precursor
-        activation: dict, optional
-            Parameters forwarded to :meth:`PrecursorBuilder.activation`
+        activation: list, optional
+            A list of parameters describing the ion activation method used.
         isolation_window_args: tuple, list, or dict, optional
-            Parameters forwarded to :meth:PrecursorBuilder.isolation_window`,
+            Parameters forwarded to :meth:`PrecursorBuilder.isolation_window`,
             tuple or list values are converted into :class:`dict` of the correct
-            structure.
+            structure. This argument may also be passed as `isolation_window`.
         params: list, optional
-            The cv-params of the first selected ion
+            The cvParams of the first selected ion
         intensity_unit: str
             The intensity unit of the first selected ion
         scan_id: str, optional
@@ -811,6 +853,12 @@ class PlainMzMLWriter(ComponentDispatcher, XMLDocumentWriter):
         '''
         if isinstance(mz, PrecursorBuilder):
             return self.Precursor(**mz.pack())
+        if isinstance(mz, dict):
+            return self.Precursor(**mz)
+        if isolation_window_args is None:
+            isolation_window_args = kwargs.get("isolation_window")
+        if mz is None:
+            mz = kwargs.get("selected_ion_mz")
         if scan_id is not None:
             spectrum_reference = scan_id
         if params is None:
@@ -823,7 +871,10 @@ class PlainMzMLWriter(ComponentDispatcher, XMLDocumentWriter):
         else:
             ion_list = None
         if isolation_window_args:
-            isolation_window_tag = self.IsolationWindow(**isolation_window_args)
+            if isinstance(isolation_window_args, (list, tuple)):
+                isolation_window_tag = self.IsolationWindow(*isolation_window_args)
+            else:
+                isolation_window_tag = self.IsolationWindow(**isolation_window_args)
         else:
             isolation_window_tag = None
         precursor = self.Precursor(
@@ -836,8 +887,8 @@ class PlainMzMLWriter(ComponentDispatcher, XMLDocumentWriter):
     def precursor_builder(self, mz=None, intensity=None, charge=None, spectrum_reference=None, activation=None,
                           isolation_window_args=None, params=None,
                           intensity_unit=DEFAULT_INTENSITY_UNIT, scan_id=None,
-                          external_spectrum_id=None,
-                          source_file_reference=None):
+                          external_spectrum_id=None, source_file_reference=None,
+                          isolation_window=None):
         '''Create a :class:`PrecursorBuilder`, an object to help populate the precursor information
         data structure.
 
@@ -858,12 +909,13 @@ class PlainMzMLWriter(ComponentDispatcher, XMLDocumentWriter):
         activation: dict or list, optional
             Parameters forwarded to :meth:`PrecursorBuilder.activation`. This should be a dictionary
             with a key "params" and a list of :class:`~.CVParam` coerce-able values, with additional
-            optional keys naming other :class:`~.CVParam` coerce-able values.
+            optional keys naming other :class:`~.CVParam` coerce-able values. If a :class:`list` is
+            passed, it will be wrapped in one e.g. ``{"params": activation}``
         isolation_window_args: tuple, list, or dict, optional
             Parameters forwarded to :meth:PrecursorBuilder.isolation_window`,
             tuple or list of three values are converted into :class:`dict` of the correct
             structure. The expected keys are "lower", the lower m/z offset, "target", the center m/z,
-            and "upper", the upper m/z offset.
+            and "upper", the upper m/z offset. You may also pass this argumemt as `isolation_window`.
         params: list, optional
             The cv- and user-params of the first selected ion, in addition to `mz`, `intensity`,
             `charge`.
@@ -880,6 +932,8 @@ class PlainMzMLWriter(ComponentDispatcher, XMLDocumentWriter):
         -------
         :class:`PrecursorBuilder`
         '''
+        if isolation_window_args is None:
+            isolation_window_args = isolation_window
         if scan_id is None:
             spectrum_reference = scan_id
         inst = PrecursorBuilder(
@@ -887,7 +941,7 @@ class PlainMzMLWriter(ComponentDispatcher, XMLDocumentWriter):
             external_spectrum_id=external_spectrum_id)
         if mz is not None or intensity is not None or charge is not None or params is not None:
             inst.selected_ion(
-                mz=mz, intensity=intensity, charge=charge,
+                selected_ion_mz=mz, intensity=intensity, charge=charge,
                 intensity_unit=intensity_unit, params=params)
         if isolation_window_args is None:
             if isinstance(isolation_window_args, (tuple, list)):
@@ -897,6 +951,8 @@ class PlainMzMLWriter(ComponentDispatcher, XMLDocumentWriter):
                     "upper": isolation_window_args[2]}
             inst.isolation_window(isolation_window_args)
         if activation is not None:
+            if isinstance(activation, (list, tuple)):
+                activation = {'params': activation}
             inst.activation(activation)
         return  inst
 
@@ -949,6 +1005,28 @@ class PrecursorBuilder(ElementBuilder):
 
 
 class IndexedMzMLWriter(PlainMzMLWriter):
+    """A high level API for generating indexed mzML XML files from simple Python objects.
+
+    This class depends heavily on :mod:`lxml`'s incremental file writing API which in turn
+    depends heavily on context managers. Almost all logic is handled inside a context
+    manager and in the context of a particular document. Since all operations assume
+    that they have access to a universal identity map for each element in the document,
+    that map is centralized in this class.
+
+    `MzMLWriter` inherits from :class:`.ComponentDispatcher`, giving it a :attr:`context`
+    attribute and access to all `Component` objects pre-bound to that context with attribute-access
+    notation.
+
+    Attributes
+    ----------
+    chromatogram_count : int
+        A count of the number of chromatograms written
+    spectrum_count : int
+        A count of the number of spectra written
+    index_builder : :class:`~.IndexingStream`
+        A writing stream that automatically tokenizes and records byte offsets for
+        specific XML tags.
+    """
     def __init__(self, outfile, close=False, vocabularies=None, missing_reference_is_error=False,
                  vocabulary_resolver=None, id=None, accession=None, **kwargs):
         outfile = IndexingStream(outfile)
