@@ -1,10 +1,20 @@
 import os
 import tempfile
+import gzip
 
 import pytest
 
 fixtured_files = []
 
+
+compression_openers = [open, gzip.GzipFile]
+
+
+@pytest.fixture(scope='function', params=compression_openers)
+def compressor(request):
+    return request.param
+
+
 @pytest.fixture(scope='function')
 def output_path(request):
     fd, path = tempfile.mkstemp()
@@ -13,13 +23,12 @@ def output_path(request):
     def fin():
         try:
             os.remove(path)
-        except OSError:
-            pass
+        except OSError as e:
+            print(e)
     request.addfinalizer(fin)
     return path
 
 
-
 test_root = os.path.abspath(os.path.dirname(__file__))
 
 
